#!/usr/bin/python
################################################################################
#	 This is a Pyla form module.
#    Copyright (C) 2003 Andrea Nicolini
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#	 Andrea Nicolini's Address is:
#	 
#		Via G.Zoccoli n.80 Baggiovara 
#		Modena, ITALY 
#		E-Mail <andrea@teamsw.it>
#
################################################################################

import string,os,re,shelve,time,tooltip

from Tkinter import *
from tkMessageBox import *
from tkFileDialog import askopenfilename,asksaveasfilename

config = {}
langhash = {}
abfile = ''

class VEntry(Entry):

	def __init__(self, master, maxlength=10, **args):
		Entry.__init__(self,master,args)	
		self.maxlength = maxlength
		self.var = StringVar()
		self.var.trace('w',self.validate)
		self.config(textvariable=self.var)
		
	def validate(self,*dummy):
		str = self.get()
		if len(str) > self.maxlength-1:
			self.var.set(str[:self.maxlength-1])

class REntry(Entry):

	def __init__(self, master, chars,**args):
		Entry.__init__(self,master,args)	
		self.var = StringVar()
		self.chars = chars
		self.var.trace('w',self.validate)
		self.config(textvariable=self.var)
		
	def validate(self,*dummy):
		str = self.get()
		for c in str:
			if c not in self.chars:
				self.var.set(str[:-1])
				break


class SendFaxFrame(Toplevel):

	is_OK = None
	cover_only = 0
	faxnumber = ''
	sendhash = {}
	sendtime = 0
	expiretime = 0
	priority = 0
	maxdials = 0
	cover_to = ''
	cover_from = ''
	cover_to_company = ''
	cover_regarding = ''
	cover_path = ''
	jobinfo = ''
	modem = ''
	archive = None
	adrbook = None

	def __init__(self,parent=None,backuphash={},conly=0,is_tscm=0,fnum=0):
		self.is_OK = 0
		self.sendhash = {}
		self.backuphash = backuphash
		self.tscm = is_tscm
		self.fnum = fnum
		self.cover_only = conly
		Toplevel.__init__(self,parent)
		self.resizable(0,0)
		self.title(langhash['sendfaxtitle'])
		self.makewidgets()
		self.focus_set()
		self.grab_set()
		self.wait_window()
		

	def makewidgets(self):
	
		self.framebot = Frame(self,borderwidth=2,relief=GROOVE)
		self.framebot.pack(side=BOTTOM,fill=X)
		self.framesx = Frame(self)
		self.framesx.pack(side=LEFT,fill=BOTH,expand=Y,anchor=N)

		Label(self.framesx,text=langhash['sendfaxopt'],relief=RIDGE,font=("Helvetica",12,"bold","italic")).pack(fill=X)
		frm = Frame(self.framesx,borderwidth=2,relief=GROOVE)
		frm.pack(side=TOP,fill=X)

		tfrm = Frame(frm)
		lab = Label(tfrm,text=langhash['sendfaxadr'],width=12,anchor=W,font=("Helvetica",10,"bold"))
		lab.pack(side=LEFT)
		tfrm.pack(side=TOP,fill=X)
	
		self.fax = Entry(tfrm,bg="white")
		self.fax.focus()
		self.fax.bind('<Return>',self.add_address)
		self.fax.pack(side=LEFT,fill=X,expand=Y)
		
		lfrm = Frame(frm)
		lfrm.pack(side=TOP,fill=X)
		llab = Label(lfrm,text=langhash['sendfaxsel'],width=12,anchor=W,font=("Helvetica",10,"bold"))
		llab.pack(side=LEFT)
                self.list = Listbox(lfrm,height=3,width=15,bg="White",selectmode=SINGLE,relief=SUNKEN)

                sbar = Scrollbar(lfrm)
                sbar.config(command=self.list.yview)
                self.list.config(yscrollcommand=sbar.set)
		self.list.pack(side=LEFT,fill=X,expand=Y)
		sbar.pack(side=RIGHT,fill=Y)
		self.list.bind('<ButtonRelease-1>',self.cur_cover)

		self.imgAdrbook = PhotoImage(file=os.path.join(gifdir,"adrbook.gif"))
		self.imgAddAdr = PhotoImage(file=os.path.join(gifdir,"addadr.gif"))
		self.imgDelAdr = PhotoImage(file=os.path.join(gifdir,"deladr.gif"))
		self.imgImpBku = PhotoImage(file=os.path.join(gifdir,"imparr.gif"))

		self.addAdr = Button(tfrm,text="AddAddress",image=self.imgAddAdr,command=self.add_address)
		self.addAdr.pack(side=LEFT,fill=X)
		tooltip.ToolTip(self.addAdr,langhash['tipaddadr'])
		
		self.delAdr = Button(tfrm,text="DelAddress",image=self.imgDelAdr,command=self.del_address)
		self.delAdr.pack(side=LEFT,fill=X)
		tooltip.ToolTip(self.delAdr,langhash['tipdeladr'])

		if self.tscm:
			self.impBku = Button(tfrm,text="Import Backup",image=self.imgImpBku,command=self.import_backup)
			self.impBku.pack(side=LEFT)
			tooltip.ToolTip(self.impBku,langhash['tiplatestsend'])

		self.Adrbook = Button(tfrm,text="Addressbook",image=self.imgAdrbook,command=self.address_book)
		self.Adrbook.pack(side=LEFT,fill=X)
		tooltip.ToolTip(self.Adrbook,langhash['tipaddressbook'])
		
		frmcover = Frame(self.framesx,borderwidth=2,relief=GROOVE)
		frmcover.pack(expand=Y,fill=X)
		frmcov = Frame(frmcover)
		frmcov.pack(expand=Y,fill=X)
		Label(frmcov,text=langhash['sendfaxusecov'],width=12,anchor=W,font=("Helvetica",10,"bold")).pack(side=LEFT)

		self.covopthash = {}
		coverlist = []
		if config['faxcover']:
			fhash = eval(config['faxcover'])
			for k in fhash.keys():
				if self.tscm:
					self.covopthash[k] = fhash[k]['path']
					coverlist.append(k)
				else:
					if string.split(fhash[k]['path'],':')[0] != 'tscm':
						self.covopthash[k] = fhash[k]['path']
						coverlist.append(k)
			
		self.covnamevar = StringVar()
		self.covopt = apply(OptionMenu,(frmcov,self.covnamevar,langhash['covernone'])+tuple(coverlist))
		self.covopt.pack(side=LEFT)
	
		if config['default_cover'] and config['default_cover'] in coverlist:
			self.covnamevar.set(config['default_cover'])
		else:
			self.covnamevar.set(langhash['covernone'])

		if self.tscm:
			
			self.imgleftArr = PhotoImage(file=os.path.join(gifdir,"leftarr.gif"))
			self.imgrightArr = PhotoImage(file=os.path.join(gifdir,"rightarr.gif"))


			self.savPre = Button(frmcov,text="Previous cover",image=self.imgleftArr,command=self.prev_cover)
			self.savPre.pack(side=LEFT)
			tooltip.ToolTip(self.savPre,langhash['tipprevcov'])

			self.comvar = IntVar()
			self.chkcov = Checkbutton(frmcov,text=langhash['sendfaxchk0'],indicatoron=0,variable=self.comvar)
			self.chkcov.pack(side=LEFT,fill=X)
			self.comvar.trace('w',self.set_commoncov)

			self.savNex = Button(frmcov,text="Next cover",image=self.imgrightArr,command=self.next_cover)
			self.savNex.pack(side=LEFT)
			tooltip.ToolTip(self.savNex,langhash['tipnextcov'])
		
		self.cof = self.create_field(frmcover,langhash['sendfaxfrom'],'entry')
		self.coc = self.create_field(frmcover,langhash['sendfaxcom'],'entry')
		self.cot = self.create_field(frmcover,langhash['sendfaxto'],'entry')
		self.cor = self.create_field(frmcover,langhash['sendfaxsub'],'entry')
		self.com = self.create_field(frmcover,langhash['sendfaxbod'],'text')
	
		frmm = Frame(self.framesx,borderwidth=2,relief=GROOVE)
		frmm.pack(side=TOP,fill=X)

		self.arcvar = StringVar()
		Label(frmm,text=langhash['sendfaxarc'],font=("Helvetica",10,"bold")).pack(side=LEFT)
		self.arcopt = OptionMenu(frmm,self.arcvar,langhash['archiveopt0'],langhash['archiveopt1'])
		self.arcopt.pack(side=LEFT)

		if int(config['archive']):
			self.arcvar.set(langhash['archiveopt1'])
		else:
			self.arcvar.set(langhash['archiveopt0'])

		self.mdmvar = StringVar()
		Label(frmm,text=langhash['sendfaxdev'],font=("Helvetica",10,"bold")).pack(side=LEFT)
		if string.strip(config['modemlist']):
			ml = string.split(config['modemlist'])
			self.mdmopt = apply(OptionMenu,(frmm,self.mdmvar)+tuple(ml))
			self.mdmvar.set(ml[0])
		else:
			self.mdmopt = apply(OptionMenu,(frmm,self.mdmvar)+('Any',))
			self.mdmvar.set('Any')
			
		self.mdmopt.pack(side=LEFT)

		frmslide = Frame(self.framesx,borderwidth=2,relief=GROOVE)
		frmslide.pack(side=TOP,fill=X)

		frmopt = Frame(frmslide)
		frmopt.pack(side=TOP,fill=X)
		self.snd = Scale(frmopt,from_=0,to=10080,troughcolor='grey',orient='horizontal',resolution=1,showvalue=0,font=("Helvetica",10,"bold"),command=self.scale_labsnd)
		self.snd.pack(side=LEFT,fill=X,expand=Y)

		self.pri = Scale(frmopt,from_=1,to=3,troughcolor='grey',orient='horizontal',resolution=1,showvalue=0,font=("Helvetica",10,"bold"),command=self.scale_labpri)
		self.pri.set(2)
		self.pri.pack(side=LEFT,fill=X,expand=Y)

		frmopt2 = Frame(frmslide)
		frmopt2.pack(side=TOP,fill=X)

		self.exp = Scale(frmopt2,from_=60,to=2880,troughcolor='grey',orient='horizontal',resolution=60,showvalue=0,font=("Helvetica",10,"bold"),command=self.scale_labexp)
		self.exp.pack(side=RIGHT,fill=X,expand=Y)
		self.exp.set(1440)

		self.dia = Scale(frmopt2,from_=1,to=24,troughcolor='grey',orient='horizontal',resolution=1,showvalue=0,font=("Helvetica",10,"bold"),command=self.scale_labdia)
		self.dia.set(int(config['maxdials']))
		self.dia.pack(side=RIGHT,fill=X,expand=Y)
	
		if self.fnum > 1:
			sndbuttext = langhash['sendfaxsend']+' (' + str(self.fnum) + ')'
		else:
			sndbuttext = langhash['sendfaxsend']

		self.sndbut = Button(self.framebot,text=sndbuttext,font=("Helvetica",10,"bold"),width=15,command=self.send)
		self.SelectCover()
		self.covnamevar.trace('w',self.SelectCover)
		self.sndbut.pack(side=LEFT,fill=X,expand=Y)

		self.quebut = Button(self.framebot,text=langhash['sendfaxqueue'],font=("Helvetica",10,"bold"),width=15,command=self.queue)
		if self.cover_only:
			self.quebut.config(state=DISABLED)
		self.quebut.pack(side=LEFT)

		Button(self.framebot,text=langhash['sendfaxcancel'],font=("Helvetica",10,"bold"),width=15,command=self.quit).pack(side=RIGHT)
		

	
	def create_field(self,parent,label='label:',type='entry'):
		if type == 'entry':
			frm = Frame(parent)
			lab = Label(frm,text=label,width=12,anchor=W,font=('helvetica',10,'bold'))
			obj = VEntry(frm,bg="white",width=40,maxlength=40,font=('courier',10))
			frm.pack(side=TOP,fill=X)
			lab.pack(side=LEFT)
			obj.pack(side=LEFT,fill=X)
		elif type == 'label':
			frm = Frame(parent)
			obj = Label(frm,text=label,font=('helvetica',10,'bold'))
			frm.pack(side=TOP,fill=X)
			obj.pack()
		elif type == 'text':
			frm = Frame(parent)
			frm2 = Frame(parent)
			lab = Label(frm,text=label,width=12,anchor=W,font=('helvetica',10,'bold'))
			obj = Text(frm2,width=60,height=5,wrap=WORD,bg="white",font=('courier',10))
			frm.pack(side=TOP,fill=X)
			frm2.pack(side=TOP,fill=X)
			lab.pack()
			obj.pack(fill=X)

		return obj
		
	def scale_labsnd(self,event):
		offset = self.snd.get()
		if offset:
			t = time.strftime(langhash['sendfaxtime0'],time.localtime(time.time()+(offset*60)))
		else: 
			t = langhash['sendfaxtime1']
		self.snd.config(label=t)

	def scale_labpri(self,event):
		offset = self.pri.get()
		if offset == 1:
			t = langhash['sendfaxpri0']
		elif offset == 2:
			t = langhash['sendfaxpri1']
		elif offset == 3:
			t = langhash['sendfaxpri2']
		self.pri.config(label=t)

	def scale_labdia(self,event):
		offset = self.dia.get()
		t = langhash['sendfaxdia'] % offset
		self.dia.config(label=t)

	def scale_labexp(self,event):
		offset = self.exp.get()/60
		if offset:
			t = langhash['sendfaxexp0'] % offset
		else: 
			t = langhash['sendfaxexp1']
		self.exp.config(label=t)

	def SelectCover(self,*dummy):
		
		if self.sendhash and self.covnamevar.get() != langhash['covernone'] and self.covopthash and\
			(string.split(self.covopthash[self.covnamevar.get()],':')[0] == 'tscm' or\
			os.access(self.covopthash[self.covnamevar.get()],os.R_OK)): 

			self.cover_path = self.covopthash[self.covnamevar.get()]
			if self.tscm:
				if string.split(self.covopthash[self.covnamevar.get()],':')[0] == 'tscm':
					self.chkcov.config(state=NORMAL)
					self.comvar.set(0)
				else:
					self.comvar.set(1)
					self.chkcov.config(state=DISABLED)

			self.cof.config(state=NORMAL,bg="white")
			self.cot.config(state=NORMAL,bg="white")
			self.coc.config(state=NORMAL,bg="white")
			self.cor.config(state=NORMAL,bg="white")
			self.com.config(state=NORMAL,bg="white")
			if self.cover_only:
				self.sndbut.config(state=NORMAL)
		else:
			self.cover_path = ''
			if self.tscm:
				self.chkcov.config(state=DISABLED)
				self.comvar.set(1)
			self.cof.config(state=DISABLED,bg="lightgray")
			self.cot.config(state=DISABLED,bg="lightgray")
			self.coc.config(state=DISABLED,bg="lightgray")
			self.cor.config(state=DISABLED,bg="lightgray")
			self.com.config(state=DISABLED,bg="lightgray")
			if self.cover_only:
				self.sndbut.config(state=DISABLED)
		
		if self.tscm:
			self.set_commoncov()
			
	def import_backup(self):
		if askyesno(langhash['askimplab'],langhash['askimport'],parent=self):
			self.sendhash = self.backuphash.copy()
			self.cof.config(state=NORMAL,bg="white")
			self.cot.config(state=NORMAL,bg="white")
			self.coc.config(state=NORMAL,bg="white")
			self.cor.config(state=NORMAL,bg="white")
			self.com.config(state=NORMAL,bg="white")
			self.comvar.set(0)
			self.ref_address()
			self.SelectCover()
	
	def address_book(self):
	
		self.adrbook = AddressBookFrame(abfile,self.sendhash,is_tscm=self.tscm)
		self.sendhash = self.adrbook.sendhash
		self.SelectCover()
		self.ref_address()
		
	def add_address(self,event=None):

		buf = string.strip(self.fax.get())
		if buf:
			if '@' in buf and self.tscm:
				self.sendhash['Manual:mail:'+buf] = { 'mail':buf }
			else:
				self.sendhash['Manual:fax:'+buf] = { 'facsimiletelephonenumber':buf }

			self.fax.delete(0, END)
			self.SelectCover()
			self.ref_address()

	def del_address(self):
	
		cursel = self.list.curselection()
                if cursel:
			selected = self.list.get(cursel[0])
                        del self.sendhash[selected]
			self.ref_address()
			self.SelectCover()
			
	def ref_address(self):

		self.save_cover()
		self.list.delete(0, END)
		keys =  self.sendhash.keys()
		keys.sort()
		for k in keys:
			self.list.insert(END,k)
                       	addrname = string.split(k,':')[-1]
                       	self.sendhash[k]['covercompany'] = addrname
                       	if not self.sendhash[k].has_key('coverfrom'):
                       		self.sendhash[k]['coverfrom'] = config['userdesc']

		s = self.list.size()
		self.list.selection_clear(0,END)
		self.list.selection_set(s-1)
		self.list.see(s-1)
		self.set_coverfields()
	
	def set_commoncov(self,event=None,*dummy):
		if self.comvar.get():
			self.savPre.config(state=DISABLED)
			self.savNex.config(state=DISABLED)
		else:
			self.savPre.config(state=NORMAL)
			self.savNex.config(state=NORMAL)
		
	def set_coverfields(self,event=None):
	
		selected = self.list.curselection()

		if self.tscm and selected and not self.comvar.get():
			sel = int(selected[0])
			self.list.activate(sel)
			label = self.list.get(sel)
                     	self.cof.delete(0,END)
                    	self.cot.delete(0,END)
       	             	self.coc.delete(0,END)
                     	self.cor.delete(0,END)
                     	self.com.delete("1.0",END)

			if self.sendhash[label].has_key('covercompany'):
                       		self.coc.insert(0,self.sendhash[label]['covercompany'])
			if self.sendhash[label].has_key('coverfrom'):
                       		self.cof.insert(0,self.sendhash[label]['coverfrom'])
			if self.sendhash[label].has_key('coverto'):
                    	   	self.cot.insert(0,self.sendhash[label]['coverto'])
			if self.sendhash[label].has_key('coverregarding'):
                       		self.cor.insert(0,self.sendhash[label]['coverregarding'])
			if self.sendhash[label].has_key('covercomments'):
                       		self.com.insert(END,self.sendhash[label]['covercomments'])

	def cur_cover(self,event=None):
		self.save_cover()
		self.set_coverfields()

	def prev_cover(self,event=None):

		self.save_cover()
		curind = self.list.index(ACTIVE) 
		if curind == 0:
			ind = self.list.size() - 1
		else:
			ind = curind - 1
		self.list.activate(ind)
		self.list.selection_clear(0,END)
		self.list.selection_set(ind)
		self.list.see(ind)
		self.set_coverfields()
	
	def next_cover(self,event=None):
	
		self.save_cover()
		curind = self.list.index(ACTIVE) 
		if curind == self.list.size() -1:
			ind = 0
		else:
			ind = curind + 1
		self.list.activate(ind)
		self.list.selection_clear(0,END)
		self.list.selection_set(ind)
		self.list.see(ind)
		self.set_coverfields()
		

	def save_cover(self,event=None):

		label = self.list.get(ACTIVE)
		if self.tscm and label and self.sendhash.has_key(label) and not self.comvar.get():
			self.sendhash[label]['coverfrom'] = self.cof.get()
			self.sendhash[label]['coverto'] = self.cot.get()
			self.sendhash[label]['covercompany'] = self.coc.get()
			self.sendhash[label]['coverregarding'] = self.cor.get()
			self.sendhash[label]['covercomments'] = self.txtwrap(self.com.get("1.0",END),60)

	def send(self,event=None):

		self.faxnumber = self.fax.get()
		
		if self.sendhash:
			if self.cover_path:
				self.is_OK = 2
			else:
				self.is_OK = 1
				
			self.expiretime = self.exp.get()
			self.maxdials = self.dia.get()
			self.sendtime = self.snd.get()
			self.priority = self.pri.get()
			self.archive = self.arcvar.get()
			self.modem = self.mdmvar.get()
			self.cover_from = self.cof.get()
			self.cover_to = self.cot.get()
			self.cover_to_company = self.coc.get()
			self.cover_regarding = self.cor.get()
			#self.cover_comments = self.textfmt(self.com.get("1.0",END))
			self.cover_comments = self.txtwrap(self.com.get("1.0",END),60)

			ji_re = string.strip(self.cor.get()) 
			ji = ji_re 
			self.jobinfo = unicode(ji).encode('iso-8859-1')
		
			if len(string.split(self.cover_comments,'\n')) > 20:
				self.is_OK = 0
				ErrorBox(self,langhash['covererror1'],langhash['covererror2'])
			else:
				self.save_cover()
				self.quit()
		else:
			ErrorBox(self,langhash['numbererror0'],langhash['numbererror1'])
		
	def textfmt(self,buf):
		i = 0
		tmp = ''
		lines = []
		for c in buf:
			if c == '\n':
				if string.strip(tmp):
					lines.append(tmp)
				tmp = ''
				i = 0 
			elif i == 59:
				tmp += c
				lines.append(tmp)
				tmp = ''
				i = 0 
			else:
				tmp += c
				i = i + 1
		return string.join(lines,'\n')
	
	def txtwrap(self,text, width):
    		return reduce(lambda line, word, width=width: '%s%s%s' %
                  (line,
                   ' \n'[(len(line[line.rfind('\n')+1:])
                         + len(word.split('\n',1)[0]
                              ) >= width)],
                   word),
                  text.split(' ')
                 )

	def queue(self):
		self.is_OK = 3
		self.destroy()

	def quit(self):
		self.destroy()


class ConfigurationFrame(Toplevel):

	is_OK = None

	def __init__(self,config,is_tscm=0):
		Toplevel.__init__(self)
		self.tscm = is_tscm
		self.cfghash = config
		self.resizable(0,0)
		self.title(langhash['cfgtitle'])
		self.makewidgets()
		self.focus_set()
		self.grab_set()
		self.wait_window()

	def makewidgets(self):

		self.imgBrowse = PhotoImage(file=os.path.join(gifdir,"open.gif"))
		self.imgOpenCfg = PhotoImage(file=os.path.join(gifdir,"opencfg.gif"))
	
		Label(self,text=langhash['cfgsettitle'],relief=RIDGE,font=("Helvetica",12,"italic","bold")).pack(fill=X)
		self.fxh = self.create_field(langhash['cfghost'])
		self.fxh.insert(0,self.cfghash['faxhost'])

		self.usr = self.create_field(langhash['cfguser'])
		self.usr.insert(0,self.cfghash['username'])
		
		self.usd = self.create_field(langhash['cfgusrdesc'])
		self.usd.insert(0,self.cfghash['userdesc'])

		self.psw = self.create_field(langhash['cfgpass'],'*')
		self.psw.insert(0,self.cfghash['password'])

		self.apw = self.create_field(langhash['cfgadmin'],'*')
		self.apw.insert(0,self.cfghash['adminpass'])

		self.eml = self.create_field(langhash['cfgmail'])
		self.eml.insert(0,self.cfghash['emailaddress'])
		
		self.mdm = self.create_field(langhash['cfgline'])
		self.mdm.insert(0,self.cfghash['modemlist'])

		self.cvd = self.create_field(langhash['cfgdate'])
		self.cvd.insert(0,self.cfghash['coverdate'])
 
		frmviw = Frame(self)
		frmviw.pack(side=TOP,fill=X)
		Label(frmviw,text=langhash['cfgview'],width=12,anchor=W).pack(side=LEFT)
		self.viw = Entry(frmviw,bg="white")
		self.viw.pack(side=LEFT,fill=X,expand=Y)
		Button(frmviw,text="Browse",image=self.imgBrowse,command=self.sel_viewer).pack(side=LEFT,fill=X)
		self.viw.insert(0,self.cfghash['viewer'])
		 

		frmcov = Frame(self)
		frmcov.pack(side=TOP,fill=X)
		Label(frmcov,text=langhash['cfgcov'],width=12,anchor=W).pack(side=LEFT)
		self.cov = Label(frmcov,anchor=W)
		self.cov.pack(side=LEFT,fill=X,expand=Y)
		Button(frmcov,text="Browse",image=self.imgOpenCfg,command=self.sel_cover).pack(side=LEFT,fill=X)
		self.defcov = self.cfghash['default_cover']
		if self.defcov:
			self.cov.config(text=self.defcov)
		else:
			self.cov.config(text='???')

		frmadr = Frame(self)
		frmadr.pack(side=TOP,fill=X)
		Label(frmadr,text=langhash['cfgcon'],width=12,anchor=W).pack(side=LEFT)
		self.adr = Label(frmadr,anchor=W)
		self.adr.pack(side=LEFT,fill=X,expand=Y)
		Button(frmadr,text="Browse",image=self.imgOpenCfg,command=self.sel_adrbook).pack(side=LEFT,fill=X)
		self.defadr = self.cfghash['default_adr']
		if self.defadr:
			self.adr.config(text=self.defadr)
		else:
			self.adr.config(text='???')

                self.frmmdi = Frame(self)
		self.frmmdi.pack(side=TOP,fill=X)
		Label(self.frmmdi,text=langhash['cfgdia'],width=12,anchor=W).pack(side=LEFT)
		self.mdi = Scale(self.frmmdi,from_=1,to=24,troughcolor='grey',orient='horizontal',width=4,showvalue=0,command=self.scale_update)
		self.mdi.pack(side=LEFT,fill=X)
		self.mdi.set(int(self.cfghash['maxdials']))
		
		frmnot = Frame(self)
		frmnot.pack(side=TOP,fill=X)
		self.vnot = StringVar()
		Label(frmnot,text=langhash['cfgnot'],width=12,anchor=W).pack(side=LEFT)
		if not self.tscm:
			Radiobutton(frmnot,text=langhash['cfgradio0'],variable=self.vnot,value='"none"').pack(side=LEFT,fill=X)
		Radiobutton(frmnot,text=langhash['cfgradio1'],variable=self.vnot,value='"done"').pack(side=LEFT,fill=X)
		Radiobutton(frmnot,text=langhash['cfgradio2'],variable=self.vnot,value='"done+requeue"').pack(side=LEFT,fill=X)
		if self.tscm and self.cfghash['notify'] == '"none"':
			self.vnot.set('"done"')
		else:
			self.vnot.set(self.cfghash['notify'])
	
		frmpro = Frame(self)
		frmpro.pack(side=TOP,fill=X)
		self.vpro = StringVar()
		Label(frmpro,text=langhash['cfgproto'],width=12,anchor=W).pack(side=LEFT)
		Radiobutton(frmpro,text='Hylafax',variable=self.vpro,value='hylafax').pack(side=LEFT,fill=X)
		Radiobutton(frmpro,text='Tscm',variable=self.vpro,value='tscm').pack(side=LEFT,fill=X)
		self.vpro.set(self.cfghash['protocol'])

		self.optfrm = Frame(self)
		self.optfrm.pack(side=TOP,fill=X)
		Label(self.optfrm,text=langhash['cfgopt'],width=12,anchor=W).pack(side=LEFT)
		self.arcvar = IntVar()
		self.dbgvar = IntVar()
		self.chkarc = Checkbutton(self.optfrm,text=langhash['cfgchk0'],variable=self.arcvar)
		self.chkarc.pack(side=LEFT,fill=X)
		self.arcvar.set(self.cfghash['archive'])
		self.chkdbg = Checkbutton(self.optfrm,text=langhash['cfgchk1'],variable=self.dbgvar)
		self.chkdbg.pack(side=LEFT,fill=X)
		self.dbgvar.set(self.cfghash['debug'])

		self.cdocvar = IntVar()
		self.optfrm2 = Frame(self)
		self.optfrm2.pack(side=TOP,fill=X)
		Label(self.optfrm2,text='',width=12,anchor=W).pack(side=LEFT)
		self.chkcdoc = Checkbutton(self.optfrm2,text=langhash['cfgchk2'],variable=self.cdocvar)
		self.chkcdoc.pack(side=LEFT,fill=X)
		self.cdocvar.set(self.cfghash['coverasdoc'])

		self.stsvar = IntVar()
		self.optfrm3 = Frame(self)
		self.optfrm3.pack(side=TOP,fill=X)
		Label(self.optfrm3,text='',width=12,anchor=W).pack(side=LEFT)
		self.chksts = Checkbutton(self.optfrm3,text=langhash['cfgchk3'],variable=self.stsvar)
		self.chksts.pack(side=LEFT,fill=X)
		self.stsvar.set(self.cfghash['expandsts'])

		self.adrvar = IntVar()
		self.optfrm4 = Frame(self)
		self.optfrm4.pack(side=TOP,fill=X)
		Label(self.optfrm4,text='',width=12,anchor=W).pack(side=LEFT)
		self.chkadr = Checkbutton(self.optfrm4,text=langhash['cfgchk4'],variable=self.adrvar)
		self.chkadr.pack(side=LEFT,fill=X)
		self.adrvar.set(int(self.cfghash['refonclick']))

		buttfrm = Frame(self,borderwidth=2,relief=GROOVE)
		buttfrm.pack(fill=X)
		Button(buttfrm,text=langhash['buttok'],width=10,command=self.save).pack(side=LEFT)
		Button(buttfrm,text=langhash['buttcanc'],width=10,command=self.quit).pack(side=RIGHT)
		
        def scale_update(self,event):
                offset = self.mdi.get()
                if offset == 1:
                        t = offset, langhash['cfgscaledia0']
		elif offset > 1:
                        t = offset, langhash['cfgscaledia1']
                else:
                        t = langhash['cfgscaledia2']	     	       
		self.mdi.config(label=t)

	def sel_cover(self):
		ccfg = CoverCfgFrame(self,self.cfghash)
		self.cov.config(text=self.cfghash['default_cover'])
	
	def sel_adrbook(self):
		acfg = LdapCfgFrame(self,self.cfghash)
		self.adr.config(text=self.cfghash['default_adr'])

	def sel_viewer(self):
		filename = askopenfilename()
		if filename:
			filename = os.path.normpath(filename)
			self.viw.delete(0,END)
			self.viw.insert(0,filename)

	def create_field(self,label='label:',showchar=None):
		frm = Frame(self)
		lab = Label(frm,text=label,width=12,anchor=W)
		obj = Entry(frm,bg="white",show=showchar)
		frm.pack(side=TOP,fill=X)
		lab.pack(side=LEFT)
		obj.pack(side=LEFT,fill=X)
		return obj
			
	def save(self):
		self.cfghash['faxhost'] = self.fxh.get()
		self.cfghash['username'] = self.usr.get()
		self.cfghash['userdesc'] = self.usd.get()
		self.cfghash['password'] = self.psw.get()
		self.cfghash['emailaddress'] = self.eml.get()
		self.cfghash['modemlist'] = self.mdm.get()
		self.cfghash['coverdate'] = self.cvd.get()
		self.cfghash['viewer'] = os.path.abspath(self.viw.get())
		self.cfghash['maxdials'] = self.mdi.get()
		self.cfghash['protocol'] = self.vpro.get()
		self.cfghash['notify'] = self.vnot.get()
		self.cfghash['archive'] = self.arcvar.get()
		self.cfghash['debug'] = self.dbgvar.get()
		self.cfghash['coverasdoc'] = self.cdocvar.get()
		self.cfghash['expandsts'] = self.stsvar.get()
		self.cfghash['refonclick'] = self.adrvar.get()
		self.cfghash['adminpass'] = self.apw.get()
		self.is_OK = 1
		self.quit()
		
	def quit(self):
		self.destroy()

class CfgFrame(Toplevel):

	namelist = []
	datahash = {}
	cfghash = {}
	buffer_label = ''
	default_label = ''
	label_main = ''
	buf_undo = ''
	def_undo = ''
	
	def __init__(self,parent):
		Toplevel.__init__(self)
		self.parent = parent

	def makewidgets(self):
	
		Label(self,text=self.label_main,relief=RIDGE,font=("Helvetica",12,"italic","bold")).pack(fill=X)
		
		frm = Frame(self)
		self.list = Listbox(frm,height=5,width=15,bg="White",selectmode=SINGLE,relief=SUNKEN)
		
		sbar = Scrollbar(frm)
		sbar.config(command=self.list.yview)

		self.list.config(yscrollcommand=sbar.set)
		self.list.bind('<ButtonRelease-1>',self.set_fields)
	
		frm.pack(side=TOP,fill=BOTH,expand=Y)
		sbar.pack(side=RIGHT,fill=Y)
		self.list.pack(side=TOP,expand=Y,fill=BOTH)
		buttfrm = Frame(self)
		buttfrm.pack(fill=X)
		Button(buttfrm,text=langhash['buttaddmod'],command=self.save).pack(side=LEFT)
		Button(buttfrm,text=langhash['butttoggle'],command=self.setdefault).pack(side=LEFT,fill=X,expand=Y)
		Button(buttfrm,text=langhash['buttdelete'],command=self.delete).pack(side=RIGHT)

	def makebuttons(self):

		buttfrm2 = Frame(self,borderwidth=2,relief=GROOVE)
		buttfrm2.pack(fill=X)
		Button(buttfrm2,text=langhash['buttok'],command=self.select).pack(side=LEFT)
		Button(buttfrm2,text=langhash['buttcanc'],command=self.quit).pack(side=RIGHT)
		
	def refresh_list(self):
		self.datahash = {}
		self.namelist = []
		
		self.list.delete(0,END)
		if self.cfghash[self.buffer_label]:
			self.datahash = eval(self.cfghash[self.buffer_label])
		for k in self.datahash.keys():
			if self.cfghash[self.default_label] == k:
				k = '*' + k
			else:
				k = ' ' + k
			self.namelist.append(k[1:]) 
			self.list.insert(END,k)

	def setdefault(self):
		name = self.name.get()
		if name:
			if name != self.cfghash[self.default_label]:
				self.cfghash[self.default_label] = name
			else:
				self.cfghash[self.default_label] = ''
			self.refresh_list()
	
	def create_field(self,label='label:',showchar=None):
		frm = Frame(self)
		lab = Label(frm,text=label,width=12,anchor=W)
		obj = Entry(frm,bg="white",show=showchar)
		frm.pack(side=TOP,fill=X)
		lab.pack(side=LEFT)
		obj.pack(side=LEFT,fill=X)
		return obj

	def select(self,event=None):
		self.destroy()

	def quit(self,event=None):
		self.cfghash[self.buffer_label] = self.buf_undo
		self.cfghash[self.default_label] = self.def_undo
		self.destroy()

class CoverCfgFrame(CfgFrame):

	def __init__(self,parent,cfghash):
		Toplevel.__init__(self)
		self.parent = parent
		self.cfghash = cfghash
		self.default_label = 'default_cover'
		self.buffer_label = 'faxcover'
		self.buf_undo = self.cfghash['faxcover']
		self.def_undo = self.cfghash['default_cover']
		self.label_main = 'Fax Covers'
		self.resizable(0,0)
		self.title(langhash['cfgcovtitle'])
		self.makewidgets()
		self.makeldapentry()
		self.makebrowse()
		self.makebuttons()
		self.refresh_list()
		self.focus_set()
		self.grab_set()
		self.wait_window()

	def makeldapentry(self):
		self.name = self.create_field(langhash['cfgcovname'])
		self.frmcov = Frame(self)
		Label(self.frmcov,text=langhash['cfgcovpath'],width=12,anchor=W).pack(side=LEFT)
		self.path = Entry(self.frmcov,bg="white")
		self.frmcov.pack(side=TOP,fill=X)
		self.path.pack(side=LEFT,fill=X)


	def makebrowse(self):
		self.imgBrowse = PhotoImage(file=os.path.join(gifdir,"open.gif"))
                Button(self.frmcov,text="Browse",image=self.imgBrowse,command=self.sel_file).pack(side=LEFT,fill=X)
	
	def sel_file(self):
		filename = askopenfilename()
		
		if filename: 
			filename = os.path.normpath(filename)
			self.path.delete(0,END)
			self.path.insert(0,filename)

	def set_fields(self,event=None):
 	        selected = self.list.curselection()

		if selected:
 	        	selname = self.namelist[int(selected[0])]
			self.name.delete(0,END)
			self.path.delete(0,END)
			self.name.insert(0,selname)
			self.path.insert(0,self.datahash[selname]['path'])

	def save(self):
		name = self.name.get()
		vhash = {}
		vhash['path'] = self.path.get()

		if name :
			self.datahash[name] = vhash
			self.cfghash[self.buffer_label] = str(self.datahash)
			self.refresh_list()
	
	def delete(self):
 	        selected = self.list.curselection()

		if selected:
 	        	selname = self.namelist[int(selected[0])]
			del self.datahash[selname] 
			self.cfghash[self.buffer_label] = str(self.datahash)
			if selname == self.cfghash[self.default_label]:
				self.cfghash[self.default_label] = ''
			self.name.delete(0,END)
			self.path.delete(0,END)
			self.refresh_list()


class LdapCfgFrame(CfgFrame):

	def __init__(self,parent,cfghash):
		Toplevel.__init__(self)
		self.parent = parent
		self.cfghash = cfghash
		self.default_label = 'default_adr'
		self.buffer_label = 'addressbook'
		self.buf_undo = self.cfghash['addressbook']
		self.def_undo = self.cfghash['default_adr']
		self.label_main = 'Ldaps'
		self.resizable(0,0)
		self.title(langhash['cfgldaptitle'])
		self.makewidgets()
		self.makeldapentry()
		self.makebuttons()
		self.refresh_list()
		self.focus_set()
		self.grab_set()
		self.wait_window()

	def makeldapentry(self):
		self.name = self.create_field(langhash['cfgldapname'])
		self.host = self.create_field(langhash['cfgldaphost'])
		self.basedn = self.create_field(langhash['cfgldapbase'])
		self.luser = self.create_field(langhash['cfgldapuser'])
		self.lpass = self.create_field(langhash['cfgldappass'],'*')
	

	def set_fields(self,event=None):
 	        selected = self.list.curselection()

		if selected:
 	        	selname = self.namelist[int(selected[0])]
			self.name.delete(0,END)
			self.host.delete(0,END)
			self.basedn.delete(0,END)
			self.luser.delete(0,END)
			self.lpass.delete(0,END)
			self.name.insert(0,selname)
			self.host.insert(0,self.datahash[selname]['host'])
			self.basedn.insert(0,self.datahash[selname]['bdn'])
			self.luser.insert(0,self.datahash[selname]['lu'])
			self.lpass.insert(0,self.datahash[selname]['lp'])

	def save(self):
		name = self.name.get()
		vhash = {}
		vhash['host'] = self.host.get()
		vhash['bdn'] = self.basedn.get()
		vhash['lu'] = self.luser.get()
		vhash['lp'] = self.lpass.get()

		if name :
			self.datahash[name] = vhash
			self.cfghash[self.buffer_label] = str(self.datahash)
			self.refresh_list()
	
	def delete(self):
	        selected = self.list.curselection()

		if selected:
 	        	selname = self.namelist[int(selected[0])]
			del self.datahash[selname] 
			self.cfghash[self.buffer_label] = str(self.datahash)
			if selname == self.cfghash[self.default_label]:
				self.cfghash[self.default_label] = ''
			self.name.delete(0,END)
			self.host.delete(0,END)
			self.basedn.delete(0,END)
			self.luser.delete(0,END)
			self.lpass.delete(0,END)
			self.refresh_list()

class ProgressBar(Toplevel):

	cur_sel = 0

	def __init__(self):
		Toplevel.__init__(self)
		self.message = langhash['progress0']
		self.makewidgets()
		#self.focus_set()
		#self.grab_set()
		#self.wait_window()

	def __add__(self,add):
		self.cur_sel = self.cur_sel + add
		self.ent.select_adjust(self.cur_sel)
		if self.cur_sel < len(self.message): 
			self.update()
	
	def makewidgets(self):
		self.ent = Entry(self,bg="white")
		self.ent.insert('0',self.message)
		self.ent.pack()
		
	def quit(self):
		self.destroy()

class ResJobFrame(Toplevel):

	is_OK = None
	address = ''
	transport = ''
	duplicate = 0
	
	def __init__(self,parent=None,adr='',trn='f'):
		Toplevel.__init__(self,parent)
		self.curadr = adr
		self.curtrn = trn
		self.resizable(0,0)
		self.title(langhash['modjobtitle'])
		self.makewidgets()
		self.focus_set()
		self.grab_set()
		self.wait_window()
		

	def makewidgets(self):
	
		Label(self,text=langhash['modjobtitle'],relief=RIDGE,font=("Helvetica",12,"bold","italic")).pack(fill=X)
		frmadr = Frame(self)
		Label(frmadr,text=langhash['sendfaxadr'],width=12,anchor=W).pack(side=LEFT)
		self.adr = Entry(frmadr,bg="white")
		self.adr.insert(0,self.curadr)
		self.adr.pack(side=LEFT,fill=X,expand=Y)
		frmadr.pack(side=TOP,fill=X)
		
		frmpro = Frame(self)
		frmpro.pack(side=TOP,fill=X)
		self.vpro = StringVar()
		Label(frmpro,text=langhash['sendfaxtrn'],width=12,anchor=W).pack(side=LEFT)
		Radiobutton(frmpro,text='Fax',variable=self.vpro,value='f').pack(side=LEFT,fill=X)
		Radiobutton(frmpro,text='Mail',variable=self.vpro,value='m').pack(side=LEFT,fill=X)
		self.vpro.set(self.curtrn)

		frmdup = Frame(self)
		frmdup.pack(side=TOP,fill=X)
		Label(frmdup,text=langhash['sendfaxdup'],width=12,anchor=W).pack(side=LEFT)
		self.dupvar = IntVar()
		self.chkdup = Checkbutton(frmdup,variable=self.dupvar)
		self.chkdup.pack(side=LEFT,fill=X)

		Button(self,text=langhash['buttres'],width=15,command=self.resend).pack(side=LEFT)
		Button(self,text=langhash['buttcanc'],width=15,command=self.quit).pack(side=RIGHT)
		
	def resend(self,event=None):
		self.is_OK = 1
		self.address = self.adr.get()
		self.transport = self.vpro.get()
		self.duplicate = self.dupvar.get()
		self.quit()

	def quit(self):
		self.destroy()


class ModJobFrame(Toplevel):

	is_OK = None
	sendtime = 0
	expiretime = 0
	priority = 0
	maxdials = 0
	address = ''

	def __init__(self,parent=None,is_tscm=0):
		Toplevel.__init__(self,parent)
		self.resizable(0,0)
		self.title(langhash['modjobtitle'])
		self.tscm = is_tscm
		self.makewidgets()
		self.focus_set()
		self.grab_set()
		self.wait_window()
		

	def makewidgets(self):
	
		Label(self,text=langhash['modjobtitle'],relief=RIDGE,font=("Helvetica",12,"bold","italic")).pack(fill=X)
		frmsnd = Frame(self)
		self.snd = Scale(frmsnd,from_=0,to=10080,troughcolor='grey',orient='horizontal',resolution=1,showvalue=0,command=self.scale_labsnd)
		frmsnd.pack(side=TOP,fill=X)
		self.snd.pack(side=TOP,fill=X)
		frmpri = Frame(self)
		self.pri = Scale(frmpri,from_=1,to=3,troughcolor='grey',orient='horizontal',resolution=1,showvalue=0,command=self.scale_labpri)
		self.pri.set(2)
		self.pri.pack(side=TOP,fill=X)
		frmpri.pack(side=TOP,fill=X)

		#frmadr = Frame(self)
		#Label(frmadr,text='Address:',width=12,anchor=W).pack(side=LEFT)
		#self.adr = Entry(frmadr,bg="white")
		#self.adr.pack(side=LEFT,fill=X,expand=Y)
		#frmadr.pack(side=TOP,fill=X)

		frmdia = Frame(self)
		self.dia = Scale(frmdia,from_=1,to=24,troughcolor='grey',orient='horizontal',resolution=1,showvalue=0,command=self.scale_labdia)
		self.dia.set(12)
		self.dia.pack(side=TOP,fill=X)
		frmdia.pack(side=TOP,fill=X)

		frmexp = Frame(self)
		self.exp = Scale(frmexp,from_=60,to=2880,troughcolor='grey',orient='horizontal',resolution=60,showvalue=0,command=self.scale_labexp)
		frmexp.pack(side=TOP,fill=X)
		self.exp.pack(side=TOP,fill=X)
		self.exp.set(180)


		Button(self,text=langhash['buttok'],width=15,command=self.modify).pack(side=LEFT)
		Button(self,text=langhash['buttcanc'],width=15,command=self.quit).pack(side=RIGHT)
		
	def scale_labsnd(self,event):
		offset = self.snd.get()
		if offset:
			t = time.strftime(langhash['sendfaxtime0'],time.localtime(time.time()+(offset*60)))
		else: 
			t = langhash['sendfaxtime1']
		self.snd.config(label=t)

	def scale_labexp(self,event):
		offset = self.exp.get()/60
		if offset:
			t = langhash['sendfaxexp0'] % offset
		else: 
			t = langhash['sendfaxexp1']
		self.exp.config(label=t)

	def scale_labpri(self,event):
		offset = self.pri.get()
		if offset == 1:
			t = langhash['sendfaxpri0']
		elif offset == 2:
			t = langhash['sendfaxpri1']
		elif offset == 3:
			t = langhash['sendfaxpri2']
		self.pri.config(label=t)

	def scale_labdia(self,event):
		offset = self.dia.get()
		t = langhash['sendfaxdia'] % offset
		self.dia.config(label=t)

	def modify(self,event=None):
		self.is_OK = 1
		self.sendtime = self.snd.get()
		self.priority = self.pri.get()
		#self.address = self.adr.get()
		self.expiretime = self.exp.get()
		self.maxdials = self.dia.get()
		self.quit()

	def quit(self):
		self.destroy()

def ErrorBox(dummy,title,error):
	showerror(title,error)

class ErrorBox2(Toplevel):

	msg = ''

	def __init__(self,parent,title,error):
		Toplevel.__init__(self,parent)
		self.msg = error 
		self.parent = parent
		self.geometry("+%d+%d" % (parent.winfo_rootx()+(parent.winfo_width()/2-80),parent.winfo_rooty()+(parent.winfo_height()/2-60)))
		self.resizable(0,0)
		self.title(title)
		self.makewidgets()
		self.geometry("+%d+%d" % (self.parent.winfo_rootx()+((self.parent.winfo_width()/2)-(self.winfo_reqwidth()/2)),self.parent.winfo_rooty()+(self.parent.winfo_height()/2-60)))
		self.transient(parent)
		self.focus_set()
		self.wait_window()
		
	def makewidgets(self):
		frm = Frame(self,borderwidth=2,relief=RIDGE)
		frm.pack()
		self.img = PhotoImage(file=os.path.join(gifdir,"error.gif"))
		Button(frm,text=langhash['buttok'],width=15,command=self.quit).pack(side=BOTTOM,fill=X,expand=Y)
		Label(frm,image=self.img).pack(side=LEFT,fill=X,expand=Y)
		Label(frm,text='%s' % self.msg).pack(side=LEFT,fill=X)

	def quit(self):
		self.destroy()

class AddressBookFrame(Toplevel):

	namelist = []
	
	def __init__(self,localdb,adrhash,abtype='MyContacts',is_tscm=0):
		Toplevel.__init__(self)
		self.localdb = localdb
		self.abtype = abtype
		self.tscm = is_tscm
		self.sendhash = adrhash
		self.db = {}
		self.useldap = 0

		if config['addressbook']:
			self.useldap = 1
			try:
				import ldapclass
				self.ldapobj = ldapclass.LdapDB(debug=int(config['debug']))
			except:
				self.useldap = 0
				print 'Cannot load ldap module!!!'
			
		self.title(langhash['adrbooktitle'])
		self.makewidgets()
		self.focus_set()
                self.grab_set()
                self.refresh_list()
                self.wait_window()


	def makewidgets(self):
	
		Label(self,text=langhash['adrbooktitle'],relief=RIDGE,font=("Helvetica",12,"italic","bold")).pack(fill=X)

		findfrm = Frame(self,borderwidth=2,relief=RIDGE)
		findfrm.pack(fill=X)
		Label(findfrm,text=langhash['adrbooksearch']).pack(side=LEFT)
		
		self.adrhash = {}
		self.adrlist = []
		if config['addressbook']:
			self.adrhash = eval(config['addressbook'])
			self.adrlist = self.adrhash.keys()
			self.adrlist.sort()

		self.abselect = StringVar()

		tup = (langhash['adrbookmy'],)
		if self.useldap:
			tup += tuple(self.adrlist)
		self.covopt = apply(OptionMenu,(findfrm,self.abselect)+tuple(tup))
		self.covopt.pack(side=LEFT)
		if config['default_adr']:
			self.abselect.set(config['default_adr'])
		else:
			self.abselect.set(langhash['adrbookmy'])
		
		self.abselect.trace('w',self.refresh_wrap)

		self.find = Entry(findfrm,width=10,bg="white")
		self.find.pack(side=LEFT,fill=X,expand=Y)
		self.find.bind('<Return>',self.refresh_list)
		self.imgsrc = PhotoImage(file=os.path.join(gifdir,"search.gif"))
		self.butsearch = Button(findfrm,relief=FLAT,anchor=N,image=self.imgsrc,command=self.refresh_list)
		self.butsearch.pack(side=LEFT,fill=X)
		tooltip.ToolTip(self.butsearch,langhash['tipsearchadr'])

		self.ldaplim = Scale(findfrm,from_=0,to=10000,troughcolor='grey',orient='horizontal',width=4,resolution=100,showvalue=0,command=self.scale_limit)
		self.ldaplim.pack(side=LEFT,fill=X)
		self.ldaplim.set(200)
		tooltip.ToolTip(self.ldaplim,langhash['tiplimitadr'])

		frm = Frame(self)
		frm.pack(fill=BOTH,expand=Y)

		frmlist = Frame(frm)
		frmlist.pack(side=LEFT,fill=BOTH,expand=Y)
		Label(frmlist,text=langhash['adrlab0'],relief=GROOVE,font=("Helvetica",10,"italic")).pack(fill=X)
		sbar = Scrollbar(frmlist)
		sbar.pack(side=LEFT,fill=Y)
		self.list = Listbox(frmlist,height=8,width=40,bg="White",font=("Helvetica",10),selectmode=EXTENDED,relief=SUNKEN)
		self.list.pack(side=LEFT,expand=Y,fill=BOTH)

		midbutfrm = Frame(frm)
		midbutfrm.pack(side=LEFT,fill=Y)
		self.selbut = Label(midbutfrm,text='+',relief=GROOVE,font=("Helvetica",10))
		self.selbut.pack(fill=X)
		self.selbut.bind('<1>',self.select_all)
		tooltip.ToolTip(self.selbut,"Select/Deselect all")

		self.imgAf = PhotoImage(file=os.path.join(gifdir,"addadrf.gif"))
		self.imgAm = PhotoImage(file=os.path.join(gifdir,"addadrm.gif"))
		self.imgAa = PhotoImage(file=os.path.join(gifdir,"addadra.gif"))
		self.imgD = PhotoImage(file=os.path.join(gifdir,"adrdel.gif"))

 	        self.addarrf = Button(midbutfrm,relief=GROOVE,image=self.imgAf,command=self.add_fax)
		self.addarrf.pack(side=TOP,fill=Y,expand=Y)
		tooltip.ToolTip(self.addarrf,langhash['tipaddfax'])

		if self.tscm:
			self.addarrm = Button(midbutfrm,relief=GROOVE,image=self.imgAm,command=self.add_mail)
			self.addarrm.pack(side=TOP,fill=Y,expand=Y)
			tooltip.ToolTip(self.addarrm,langhash['tipaddmail'])

			self.addarra = Button(midbutfrm,relief=GROOVE,image=self.imgAa,command=self.add_auto)
			self.addarra.pack(side=TOP,fill=Y,expand=Y)
			tooltip.ToolTip(self.addarra,langhash['tipaddauto'])

		self.delarr = Button(midbutfrm,relief=GROOVE,image=self.imgD,command=self.del_from_sendlist)
		self.delarr.pack(side=BOTTOM,fill=Y,expand=Y)
		tooltip.ToolTip(self.delarr,langhash['tipdelrcp'])

		frmlist2 = Frame(frm)
		frmlist2.pack(side=LEFT,fill=BOTH,expand=Y)
		Label(frmlist2,text=langhash['adrlab1'],relief=GROOVE,font=("Helvetica",10,"italic")).pack(fill=X)
		self.list2 = Listbox(frmlist2,height=8,width=40,bg="White",font=("Helvetica",10),selectmode=EXTENDED,relief=SUNKEN)
		self.list2.pack(side=LEFT,expand=Y,fill=BOTH)

		for k in self.sendhash.keys():
			self.list2.insert(END,k)

		sbar2 = Scrollbar(frmlist2)
		sbar2.pack(side=LEFT,fill=Y)

		sbar.config(command=self.list.yview)
		sbar2.config(command=self.list2.yview)

		self.list.config(yscrollcommand=sbar.set)
		self.list.bind('<ButtonRelease-1>',self.set_fields)
		self.list.bind('<Double-1>',self.add_fax)
		self.list2.config(yscrollcommand=sbar2.set)
		self.list2.bind('<ButtonRelease-1>',self.set_fields)
		self.list2.bind('<Double-1>',self.del_from_sendlist)
		
		frmbut = Frame(self)
		frmbut.pack(side=TOP,fill=X)
		self.imgadd = PhotoImage(file=os.path.join(gifdir,"imgadd.gif"))
		self.imgmod = PhotoImage(file=os.path.join(gifdir,"alter.gif"))
		self.imgdel = PhotoImage(file=os.path.join(gifdir,"trash.gif"))
		self.imgexp = PhotoImage(file=os.path.join(gifdir,"export.gif"))
		self.imgimp = PhotoImage(file=os.path.join(gifdir,"import.gif"))

		self.butaddr = Button(frmbut,text="Add",relief=GROOVE,image=self.imgadd,command=self.add_record)
		self.butaddr.pack(side=LEFT,fill=X,expand=Y)
		tooltip.ToolTip(self.butaddr,langhash['tipadradd'])
		self.butmodr = Button(frmbut,text="Modify",relief=GROOVE,image=self.imgmod,command=self.mod_record)
		self.butmodr.pack(side=LEFT,fill=X,expand=Y)
		tooltip.ToolTip(self.butmodr,langhash['tipadrmod'])
		self.butdelr = Button(frmbut,text="Delete",relief=GROOVE,image=self.imgdel,command=self.del_record)
		self.butdelr.pack(side=LEFT,fill=X,expand=Y)
		tooltip.ToolTip(self.butdelr,langhash['tipadrdel'])
		self.butimpb = Button(frmbut,text="Import",relief=GROOVE,image=self.imgimp,command=self.import_list)
		self.butimpb.pack(side=LEFT,fill=X,expand=Y)
		tooltip.ToolTip(self.butimpb,langhash['tipadrimp'])
		self.butexpb = Button(frmbut,text="Export",relief=GROOVE,image=self.imgexp,command=self.export_list)
		self.butexpb.pack(side=LEFT,fill=X,expand=Y)
		if not self.sendhash:
			self.butexpb.config(state=DISABLED)
		tooltip.ToolTip(self.butexpb,langhash['tipadrexp'])
		
		frmsts = Frame(self)
		frmsts.pack(side=TOP,fill=X)
		self.status = Label(frmsts)
		self.status.pack(fill=X)

		frmedit = Frame(self,borderwidth=2,relief=GROOVE)
		frmedit.pack(side=TOP,fill=X)
		
		frmname = Frame(frmedit)
		Label(frmname,text=langhash['adrname'],width=10,anchor=W).pack(side=LEFT)
		self.name = Entry(frmname,bg="white")
		frmname.pack(side=TOP,fill=X)
		self.name.pack(side=LEFT,fill=X,expand=Y)
		
		frmnum = Frame(frmedit)
		Label(frmnum,text=langhash['adrnumber'],width=10,anchor=W).pack(side=LEFT)
		self.number = REntry(frmnum,chars=string.digits+'-+/\\,;',bg="white")
		frmnum.pack(side=TOP,fill=X)
		self.number.pack(side=LEFT,fill=X,expand=Y)
		
		if self.tscm:
			frmmai = Frame(frmedit)
               	 	Label(frmmai,text=langhash['adrmail'],width=10,anchor=W).pack(side=LEFT)
	       	 	self.mail = Entry(frmmai,bg="white")
	       	 	frmmai.pack(side=TOP,fill=X)
	       	 	self.mail.pack(side=LEFT,fill=X,expand=Y)

		buttfrm = Frame(self,borderwidth=2)
		buttfrm.pack(fill=X)
		Button(buttfrm,text=langhash['buttok'],width=10,command=self.select).pack(side=LEFT)
		Button(buttfrm,text=langhash['buttcanc'],width=10,command=self.quit).pack(side=RIGHT)

	def add_auto(self,event=None):
		self.add_to_sendlist('auto')
	def add_fax(self,event=None):
		self.add_to_sendlist('fax')
	def add_mail(self,event=None):
		self.add_to_sendlist('mail')

	def scale_limit(self,event):
		offset = self.ldaplim.get()
		if offset:
			t = langhash['adrscalelim0'] % offset
		else: 
			t = langhash['adrscalelim1']
		self.ldaplim.config(label=t)

	def select_all(self,event=None):
		self.selbut.config(text='-')
		self.selbut.bind('<1>',self.deselect_all)
		self.list.select_set(0,END)

	def deselect_all(self,event=None):
		self.selbut.config(text='+')
		self.selbut.bind('<1>',self.select_all)
		self.list.select_clear(0,END)

	def refresh_wrap(self,event=None,*dummy):
		abtype = self.abselect.get()
		if abtype == langhash['adrbookmy'] or int(config['refonclick']):
			self.refresh_list()

	def refresh_list(self,event=None,*dummy):
		self.config(cursor="watch")
		self.deselect_all()
		self.update()
		self.abtype = self.abselect.get()
		if self.abtype == langhash['adrbookmy']:
			if self.tscm: self.addarra.config(state=DISABLED)
			self.refresh_local()
		elif self.adrhash.has_key(self.abtype):
			if self.tscm: self.addarra.config(state=NORMAL)
			self.refresh_ldap()
		self.config(cursor="")
		self.find.focus()

	def refresh_ldap(self):
		self.ldapdb = {}
		result = []
		search = self.find.get()
		lim = self.ldaplim.get()
		selectedab = self.abselect.get()
		selectedhost = self.adrhash[selectedab]['host']
		bdn = self.adrhash[selectedab]['bdn']
		luser = self.adrhash[selectedab]['lu']
		lpass = self.adrhash[selectedab]['lp']
		
		try:
			splithost = string.split(selectedhost,':')
			if len(splithost) > 1:
				self.ldapobj.sethost(splithost[0],int(splithost[1]))
			else:	
				self.ldapobj.sethost(selectedhost)

	       	 	self.ldapobj.login(luser,lpass,bdn)

			if search:
	   			result = self.ldapobj.query('(&(objectclass=inetorgperson)(cn=*%s*))' % search 
								,scope=2,attr=['cn','mail','facsimiletelephonenumber'],limit=lim)
			else:
	   			result = self.ldapobj.query('(&(objectclass=inetorgperson)(cn=*))' 
								,scope=2,attr=['cn','mail','facsimiletelephonenumber'],limit=lim)
	 		self.ldapobj.logout()
		except Exception,msg:
			self.status.config(text=langhash['adrerror0']+str(msg))

		self.list.delete(0,END)
		
 		self.namelist = []
                for r in result:
                        l = r[1]['cn'][0]
                        self.namelist.append(l)
                        tmph = {'facsimiletelephonenumber':'','mail':''}
                        for attr in r[1].keys():
                                lattr = string.lower(attr)
                                tmph[lattr] = r[1][attr][0]
                        self.ldapdb[l] = tmph

                self.namelist.sort()

                for l in self.namelist:
                        self.list.insert(END,l)


	def refresh_local(self):
		self.db = {}
		self.namelist = []

		if os.access(self.localdb,os.R_OK):
			f = open(self.localdb,"r")
			buf = f.readline()
			if buf:
				self.db = eval(buf)
			f.close()
		
		search = string.lower(self.find.get())
		lim = self.ldaplim.get()
		keys = self.db.keys()
		keys.sort()

		self.list.delete(0,END)

		i = 0
		for l in keys:
			ll = string.lower(l)
			if string.find(ll,search) != -1:
				self.namelist.append(l)
				self.list.insert(END,l)
				i += 1
			if i and i == lim:
				break

	def set_fields(self,event=None):

		if self.abtype == langhash['adrbookmy']:
			abdata = self.db
		else:
			abdata = self.ldapdb
		
 	        selected = self.list.curselection()
		if selected:
			self.name.delete(0,END)
                        self.name.insert(0,self.namelist[int(selected[0])])
                        self.number.delete(0,END)
                        self.number.insert(0,abdata[self.namelist[int(selected[0])]]['facsimiletelephonenumber'])
			if self.tscm:
                     	   	self.mail.delete(0,END)
                       	 	self.mail.insert(0,abdata[self.namelist[int(selected[0])]]['mail'])


	def add_to_sendlist(self,trans):

		if self.abtype == langhash['adrbookmy']:
			abdata = self.db
		else:
			abdata = self.ldapdb
		
 	        selected = self.list.curselection()

 		for i in selected:
                        name = self.namelist[int(i)]
                        label = self.abtype + ':' + trans + ':' + self.namelist[int(i)]
                        if not self.sendhash.has_key(label):
                                self.list2.insert(END,label)

                        self.sendhash[label] = abdata[name]
                        self.sendhash[label]['transport'] = trans

		if len(self.sendhash):
			self.butexpb.config(state=NORMAL)

	def del_from_sendlist(self,event=None):
		
 	        selected = self.list2.curselection()
		
		count = 0
		for i in selected:
			idx = int(i) - count
			del self.sendhash[self.list2.get(idx)]
			self.list2.delete(idx)
			count += 1 

		if not len(self.sendhash):
			self.butexpb.config(state=DISABLED)

	def add_record(self):
		name = self.name.get()
		num = self.number.get()
		if self.tscm:
			mail = self.mail.get()
		else:
			mail = ''

		if name and (num or mail):

			if self.abtype == langhash['adrbookmy']:
				if self.db.has_key(name):
					self.status.config(text=langhash['adrerror1'])
				else:
					f = open(self.localdb,'w')
                                        self.db[name] = {'cn':name,'facsimiletelephonenumber':num,'mail':mail}
                                        f.write(str(self.db))
                                        f.close()
                                        self.status.config(text=langhash['adrerror2'])
			else:
				selectedab = self.abselect.get()
				bdn = self.adrhash[selectedab]['bdn']
				luser = self.adrhash[selectedab]['lu']
				lpass = self.adrhash[selectedab]['lp']
			       	self.ldapobj.login(luser,lpass,bdn)

				h = {}
				h['cn'] = name
				h['facsimiletelephonenumber'] = [num]
				h['mail'] = [mail]
				ret = self.ldapobj.addrecord(h)
			       	self.ldapobj.logout()
				if ret:
					self.status.config(text=langhash['adrerror3']+ret[0]['desc'])
				else:
                                        self.status.config(text=langhash['adrerror2'])
		else:
                        self.status.config(text=langhash['adrerror4'])
				
		self.refresh_list()

	def mod_record(self):
		name = self.name.get()
		num = self.number.get()
		if self.tscm:
			mail = self.mail.get()
		else:
			mail = ''

                if name and (num or mail):

			if self.abtype == langhash['adrbookmy']:
				if name and (num or mail):
                                        f = open(self.localdb,'w')
                                        self.db[name] = {'cn':name,'facsimiletelephonenumber':num,'mail':mail}
                                        f.write(str(self.db))
                                        f.close()
                                        self.status.config(text=langhash['adrerror2'])

			else:
				selectedab = self.abselect.get()
				bdn = self.adrhash[selectedab]['bdn']
				luser = self.adrhash[selectedab]['lu']
				lpass = self.adrhash[selectedab]['lp']
			       	self.ldapobj.login(luser,lpass,bdn)

				h = {}
				h['cn'] = name
				if mail: h['mail'] = mail
				h['facsimiletelephonenumber'] = num
				dn = self.ldapobj.makedn('cn',h['cn'])
				ret = self.ldapobj.modrecord(dn,h)
			       	self.ldapobj.logout()
				if ret:
					self.status.config(text=langhash['adrerror5']+ret[0]['desc'])
				else:
                                        self.status.config(text=langhash['adrerror2'])
				
		else:
                        self.status.config(text=langhash['adrerror4'])


	def del_record(self):
		if self.abtype == langhash['adrbookmy']:
			for sel in self.list.curselection():
				del self.db[self.list.get(sel)] 
			f = open(self.localdb,'w')
			f.write(str(self.db))
			f.close()
		else:
			selectedab = self.abselect.get()
			bdn = self.adrhash[selectedab]['bdn']
			luser = self.adrhash[selectedab]['lu']
			lpass = self.adrhash[selectedab]['lp']
			self.ldapobj.login(luser,lpass,bdn)

			for sel in self.list.curselection():
				cn = self.list.get(sel)
				ret = self.ldapobj.delete(self.ldapobj.makedn('cn',cn))
				if ret:
					self.status.config(text=langhash['adrerror6']+ret[0]['desc'])
					break
				else:
                                        self.status.config(text=langhash['adrerror2'])
				
		       	self.ldapobj.logout()
			

		self.name.delete(0,END)
		self.number.delete(0,END)
		if self.tscm: self.mail.delete(0,END)
		self.refresh_list()

	def import_list(self):
		filename = askopenfilename(title=langhash['askimport'],filetypes=[('Adr','*.adr')])
	
		if filename and os.access(filename,os.R_OK):
			try:
				f = shelve.open(filename)
				for k in f.keys():
					self.sendhash[k] = f[k]
				f.close()
				for k in self.sendhash:
					if k not in self.list2.get(0,END):
                               		 	self.list2.insert(END,k)
					
			except:
                                self.status.config(text=langhash['adrerror7'])

			else:
                                self.status.config(text=langhash['adrerror2'])
		else:
                        self.status.config(text=langhash['adrerror8'])
		
	def export_list(self):
		if self.sendhash:
			filename = asksaveasfilename(title=langhash['askexport'],defaultextension='.adr',filetypes=[('Adr','*.adr')])
	
			if filename: 
				try:
					if os.access(filename,os.W_OK):
						os.unlink(filename)
					f = shelve.open(filename)
					for k in self.sendhash.keys():
						f[k] = self.sendhash[k]
					f.close()
				except:
                                	self.status.config(text=langhash['adrerror9'])
				else:
                                	self.status.config(text=langhash['adrerror2'])
			else:
                              	self.status.config(text=langhash['adrerror9'])
			
	def select(self,event=None):
		self.destroy()

	def quit(self,event=None):
		self.sendhash = {}
		self.destroy()

class AboutFrame(Toplevel):


	def __init__(self,parent=None):
		Toplevel.__init__(self,parent)
		self.resizable(0,0)
		self.title(langhash['aboutpyla'])
		self.makewidgets()
		self.focus_set()
		self.grab_set()
		self.wait_window()
		

	def makewidgets(self):
		abouttxt = """
This is a Pyla client for Hylafax.
Copyright (C) 2003 Andrea Nicolini

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Andrea Nicolini's Address is:
	 
Via G.Zoccoli n.80 Baggiovara 
Modena, ITALY 
E-Mail <andrea@teamsw.it>
		"""

		frm = Frame(self)
		lab = Label(frm,text=abouttxt,justify="center",relief=SUNKEN)
		frm.pack(side=TOP,fill=X)
		lab.pack(side=TOP,fill=X,expand=Y)
	
		Button(self,text=langhash['buttok'],width=15,command=self.quit).pack(side=TOP)
		
	def quit(self):
		self.destroy()

