#!/usr/bin/python 
################################################################################
#	 This is a Pyla protocol module.
#    Copyright (C) 2003 Andrea Nicolini
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#	 Andrea Nicolini's Address is:
#	 
#		Via G.Zoccoli n.80 Baggiovara 
#		Modena, ITALY 
#		E-Mail <andrea@teamsw.it>
#
################################################################################

import string,os,ftplib,xmlrpclib,StringIO,shelve

JFMT = "%j|%o|%e|%P|%a|%D|%z|%s"
pbar = None 

def get_exdesc(excpt):
	if type(excpt.args) == type(''):
		return excpt.args
	elif type(excpt.args) == type(()):
		return excpt.args[-1]
	

class Hylafax:
	#Buffer to store answers from server
	retrbuf = []
	#File to send
	filepaths = []
	#Cover to send
	coverpath = ""
	#Cover vars
	vhash = {'to':'',
		'from':'',
		'to-company':'',
		'todays-date':'',
		'page-count':'',
		'regarding':'',
		'comments':''}
	#Username
	username = "Anonymous"
	#Password
	password = ''
	#Adimistrator password
	adminpass = ''
	#Prameters hash
	parmhash = {}
	#Fax numbers
	sendhash = {}
	#Multiple cover
	multicover = 0
	#Send cover as document
	cover_as_document = 0
	#Receive temportory file
	recvfile = ''
	#Debug
	debug = 0
	#Protocol
	tscmproto = 0

	ftpexcpt = None
	filerror = None

	def __init__(self,host='',port=4559,proto='hyla',debug=0):
		self.debug = debug
		if proto == 'tscm':
			self.tscmproto = 1
		else:
			self.tscmproto = 0
			self.ftp = ftplib.FTP()
			self.ftp.passiveserver=0

		self.setfaxhost(host,port)

	def setfaxhost(self,host,port):
		hosterr = "Parameter 'host' must be a string."
		porterr = "Parameter 'port' must be an integer."

		if type(host) == type(""):
			self.faxhost = host
		else:
			raise hosterr
		if type(port) == type(1):
			self.faxport = port
		else:
			raise porterr
	
	def tscmlogin(self,username,password):

		try:
			tscm = xmlrpclib.ServerProxy('http://%s:%i' % (self.faxhost,self.faxport),verbose=self.debug)
			rights = tscm.login(username,password)
		except:
			raise 'TSCMServerError'

		if rights:
			return (tscm,rights)
		else:
			raise 'TSCMAuthError'


	def login(self,ftpobj,username,password,adminpass):
		try:
			self.ftp.set_debuglevel(self.debug)
			ftpobj.sendcmd('USER '+username)

			if password:
				ftpobj.sendcmd('PASS '+password)
			if adminpass:
				ftpobj.sendcmd('ADMIN '+adminpass)

		except ftplib.error_perm, msg:	
			raise ftplib.error_perm, msg 
		else:
			ftpobj.sendcmd('TZONE LOCAL')
			ftpobj.sendcmd('MODE S')
		
	def getstatus(self):
		if self.tscmproto:
			try:
				(tscm,rights) = self.tscmlogin(self.username,self.password) 
				status = tscm.getstatus()
				
			except 'TSCMAuthError':
				return (0,'TSCM Wrong login (status)')
			except:
				return (0,'TSCM Server error (status)')
			else:
				return (1,[status])

		else:
			try:
				self.ftp.connect(self.faxhost,self.faxport)
			except ftplib.socket.error,msg:
				msg = get_exdesc(msg)
				return (0,msg)
			else:
				try:
					self.login(self.ftp,self.username,self.password,self.adminpass)
				except ftplib.error_perm,msg:
					msg = get_exdesc(msg)
					return (0,msg)
				else:
					self.retrbuf = []
					self.ftp.retrlines('LIST status',self.callback)
					self.ftp.quit()
					return (1,self.retrbuf)

	def getqueue(self,queue,jfmt=JFMT,view='priv'):
		retlist = []
		
 		if self.tscmproto:
                        try:
                                (tscm,rights) = self.tscmlogin(self.username,self.password)
				if queue == 'recvq' and not rights['viewrcv']:
					retlist = []	
				else:
					if rights['viewall'] and view == 'pub':
						retlist = tscm.getusrjobs(queue,'*')
					else:
						retlist = tscm.getusrjobs(queue,self.username)
			
                        except 'TSCMAuthError':
                                return (0,'TSCM Wrong login (queue)')
                        except:
                                return (0,'TSCM Server error (queue)')

			else:
				return (1,retlist)

		else:
			try:
				self.ftp.connect(self.faxhost,self.faxport)
			except ftplib.socket.error,msg:
				msg = get_exdesc(msg)
				return (0,msg)
			else:
				try:
					self.login(self.ftp,self.username,self.password,self.adminpass)
				except ftplib.error_perm, msg:
					msg = get_exdesc(msg)
					return (0,msg)
				else:
					if queue == 'recvq':
						fmt = 'RCVFMT'
					else:
						fmt = 'JOBFMT'

					self.ftp.sendcmd('%s "%s"' % (fmt,jfmt))
					self.retrbuf = []
					self.ftp.retrlines('LIST '+queue,self.callback)
					self.ftp.quit()
						
					for l in self.retrbuf:
						sl = string.split(l,'|')
						if (len(sl) > 1 and sl[1] != self.username and view == 'priv') and queue != 'recvq': continue

						if len(sl) >= len(string.split(jfmt,'|')):
							retlist.append(l)
						else:	
							retlist[-1] = retlist[-1] + l + '\n'

					return (1,retlist)

	def callback(self,str):
		self.retrbuf.append(str)
		
	def jobcmd(self,cmd,job):
		ret = []
		if self.tscmproto:

                        try:
                                (tscm,rights) = self.tscmlogin(self.username,self.password)
				if rights['editall']:
					ret = tscm.jobcmd(cmd,job)
				else:
					ret = tscm.jobcmd(cmd,job,self.username)

                        except 'TSCMAuthError':
                                return (0,'TSCM Wrong login (jobcmd)')
                        except:
                                return (0,'TSCM Server error (jobcmd)')
			else:
				return ret
		else:

			joblist = []
		
			if type(job) == type([]):
				joblist = job
			else:
				joblist.append(job)
		
			ftpcmd = ftplib.FTP()
			ftpcmd.connect(self.faxhost,self.faxport)
			self.login(ftpcmd,self.username,self.password,self.adminpass)

			for j in joblist:
				try: 
					ftpcmd.sendcmd('%s %s' % (string.upper(cmd),string.strip(j)))
				except ftplib.error_perm, msg:
					ret.append('Job %s -> ' % j  + msg[0] )
				except ftplib.error_temp, msg:
					ret.append('Job %s -> ' % j  + msg[0] )
				else:
					ret.append('Job %s -> %s OK' % (j,cmd))

			ftpcmd.quit()
			return ret
	
	def setparm(self,parm,value):
		self.parmhash[string.upper(parm)] = value

	def getcovers(self):
                try:
                	(tscm,rights) = self.tscmlogin(self.username,self.password)
			chash = {}
			for c in tscm.getcoverlist():
				chash[c] = {'path':'tscm:' + c}
                except:
                	return {}
		else:
			return chash
	
	def getldaps(self):
                try:
                	(tscm,rights) = self.tscmlogin(self.username,self.password)
			chash = {}
			lhash = tscm.getldaphash()
			for l in lhash:
				host = ''
				bdn  = ''
				user = ''
				pasw = ''

				if lhash[l].has_key('port'):
					host = lhash[l]['host']+':'+str(lhash[l]['port'])
				else:
					host = lhash[l]['host']

				if lhash[l].has_key('basedn'):
					bdn = lhash[l]['basedn']

				if lhash[l].has_key('lu'):
					user = lhash[l]['lu']

				if lhash[l].has_key('lp'):
					pasw = lhash[l]['lp']

				chash[l] = {'host':host,'bdn':bdn,'lu':user,'lp':pasw}


                except:
                	return {}
		else:
			return chash
					
	def get_recvfile(self,filename):
		if self.tscmproto:
                        try:
                                (tscm,rights) = self.tscmlogin(self.username,self.password)
				ret = tscm.getdocs(filename)
				fd = open(self.recvfile,'wb')
				fd.write(ret[0].data)
				fd.close()

                        except 'TSCMAuthError':
                                return (0,'TSCM Wrong login (recvfile)')
                        except:
                                return (0,'TSCM Server error (recvfile)')

		else:	
			try:
				self.ftp.connect(self.faxhost,self.faxport)
			except ftplib.socket.error,msg:
				msg = get_exdesc(msg)
				return msg
			else:
				fd = open(self.recvfile,'wb')
				try:
					self.login(self.ftp,self.username,self.password,self.adminpass)
					self.ftp.cwd('recvq')
					self.ftp.retrbinary('RETR ' + filename,fd.write,1024)
					self.ftp.quit()
					fd.close()
				except ftplib.error_perm, msg:
					msg = get_exdesc(msg)
					return msg
				return 
			
	def resendjob(self,jobid,adr,trn,dup):
                try:
                       	(tscm,rights) = self.tscmlogin(self.username,self.password)
			if rights['editall']:
				ret = tscm.resendjob(jobid,adr,trn,dup)
			else:
				ret = tscm.resendjob(jobid,adr,trn,dup)

                except 'TSCMAuthError':
                        return (0,'TSCM Wrong login (resend)')
                except:
                        return (0,'TSCM Server error (resend)')
		else:
			return ret
		
	def alterjob(self,jobid):
		if self.tscmproto:
                        try:
                                (tscm,rights) = self.tscmlogin(self.username,self.password)
				if rights['editall']:
					ret = tscm.alterjob(jobid,self.parmhash)
				else:
					ret = tscm.alterjob(jobid,self.parmhash,self.username)

                        except 'TSCMAuthError':
                                return (0,'TSCM Wrong login (alter)')
                        except:
                                return (0,'TSCM Server error (alter)')
			else:
				return ret
		else:
			try:
				self.ftp.connect(self.faxhost,self.faxport)
			except ftplib.socket.error,err:
				return
			else:
				try:
					self.login(self.ftp,self.username,self.password,self.adminpass)
					self.ftp.sendcmd('JOB %s' % jobid)
					for jparm in self.parmhash.keys():
						self.ftp.sendcmd('JPARM %s %s' % (jparm,self.parmhash[jparm]) )
					self.ftp.sendcmd('JSUBM')
					self.ftp.quit()
				except ftplib.error_perm, msg:
					return 'Job %s -> ' % jobid  + get_exdesc(msg)
				else:
					return 'Job %s -> OK' % jobid

	def sndfax(self,ldapcfg={}):
		if self.tscmproto:
                        try:
                                (tscm,rights) = self.tscmlogin(self.username,self.password)

				jobhash = {}
				if self.username:
					jobhash['username'] = self.username
				if self.parmhash.has_key('NOTIFYADDR'):
					jobhash['usermail'] = self.parmhash['NOTIFYADDR']
				if self.parmhash.has_key('SCHEDPRI'):
					jobhash['priority'] = self.parmhash['SCHEDPRI']
				if self.parmhash.has_key('LASTTIME'):
					jobhash['expire'] =   self.parmhash['LASTTIME']
				if self.parmhash.has_key('MAXDIALS'):
					jobhash['maxdials'] = self.parmhash['MAXDIALS']
				if self.parmhash.has_key('NOTIFY'):
					jobhash['notify']   = self.parmhash['NOTIFY']

				if self.parmhash.has_key('MODEM'):
					jobhash['modemline'] = self.parmhash['MODEM']

				if self.parmhash.has_key('DONEOP') and self.parmhash['DONEOP'] == '"remove"':
					jobhash['todelete'] = 1
				else:
					jobhash['todelete'] = 0

				jobhash['jfrom'] = self.vhash['from']
				jobhash['jto'] = self.vhash['to']
				jobhash['jtocompany'] = self.vhash['to-company']
				jobhash['jsubject'] = self.vhash['regarding']
				jobhash['jbody'] = self.vhash['comments']
				if self.parmhash['SENDTIME'] != 'now':
					jobhash['sendtime'] = self.parmhash['SENDTIME']+'00'


				if self.coverpath:
                                        pagenum = 0
                                        for fil in self.filepaths:
                                        	f = file(fil)
						if f.read(4) == "%!PS":
                                        		#Find page number
                                   	     		for l in f.readlines():
                                        			if l.find('%%Pages:') != -1:
                                                                        val = l.split(':')[1].strip()
                                                       	 		if val.isdigit():
                                                                                pagenum += int(val)
                                                       	         		break
						else:
							pagenum = 0
							break
                                		f.close()
                                        self.vhash['page-count'] = str(pagenum)

				if not self.vhash['page-count'] or self.vhash['page-count'] == '0':
					self.vhash['page-count'] = '?'

				
				jobid = tscm.createjob(jobhash)
				if not jobid:
					return ['Error']
				tscmcovname = ''

				if self.coverpath:
					spath = string.split(self.coverpath,':')
					if len(spath) == 2 and spath[0] == 'tscm':
						spath = string.split(self.coverpath,':')
						if len(spath) == 2 and spath[0] == 'tscm':
							tscmcovname = spath[1]
					else:
						buf = StringIO.StringIO()
						f = CreateCover(self.coverpath,self.vhash)
						buf.write(string.join(f.readlines()))
						xfile = xmlrpclib.Binary(buf.getvalue())
						tscm.putdoc(jobid,xfile,'ps')
						buf.close()

				for k in self.sendhash.keys():
					pl = string.split(k,':')	
					a,t,n = pl[0],pl[1],pl[2]
					covhash = {}
					if tscmcovname:
						covhash = {'covername':tscmcovname}
	 					if self.multicover:
							if self.sendhash[k].has_key('coverfrom'):	
								covhash['jfrom'] = self.sendhash[k]['coverfrom']
							if self.sendhash[k].has_key('coverto'):	
								covhash['jto'] = self.sendhash[k]['coverto']
							if self.sendhash[k].has_key('covercompany'):	
								covhash['jtocompany'] = self.sendhash[k]['covercompany']
							if self.sendhash[k].has_key('coverregarding'):	
								covhash['jsubject'] = self.sendhash[k]['coverregarding']
							if self.sendhash[k].has_key('covercomments'):	
								covhash['jbody'] = self.sendhash[k]['covercomments']
						
					if t[0] == 'f':
						tscm.putadr(jobid,self.sendhash[k]['facsimiletelephonenumber'],'f',covhash)
					elif t[0] == 'm':
						tscm.putadr(jobid,self.sendhash[k]['mail'],'m',covhash)
					elif t[0] == 'a':
						ldaphost = ldapcfg[a]['host']
						ldapbasedn = ldapcfg[a]['bdn']
						tscm.putadr(jobid,self.sendhash[k]['cn'],'a',covhash,'',ldaphost,ldapbasedn)


				docok = 1
				for fil in self.filepaths:
					buf = StringIO.StringIO()
					f = file(fil,'rb')
					buf.write(f.read())
					xfile = xmlrpclib.Binary(buf.getvalue())
					f.seek(0)
					header = f.read(4)
					if header == "%!PS":
						docok = tscm.putdoc(jobid,xfile,'ps')
					elif header == "%PDF":
						docok = tscm.putdoc(jobid,xfile,'pdf')
					else:
						docok = tscm.putdoc(jobid,xfile,'fax')
					f.close()
					buf.close()
					
				if docok and tscm.submitjob(jobid):
					return [jobid]
				else:
					return ['Error']

                        except 'TSCMAuthError':
                                return (0,'TSCM Wrong login (sndfax)')
                        except:
                                return (0,'TSCM Server error (sndfax)')
		else:
			try:
				self.ftp.connect(self.faxhost,self.faxport)
			except ftplib.socket.error,err:
				return
			else:
				self.login(self.ftp,self.username,self.password,self.adminpass)
				#Send documents
				if self.coverpath:
					if self.filepaths: #Only if you have a fax besides a cover
						pagenum = 0
						for fil in self.filepaths:
							f = file(fil)
							if f.read(4) == "%!PS":
								#Find page number
								for l in f.readlines():
									if l.find('%%Pages:') != -1:
										val = l.split(':')[1].strip()
										if val.isdigit():
											pagenum += int(val)
											break
							else:
								pagenum = 0
								break
							f.close()
						self.vhash['page-count'] = str(pagenum)

					if not self.vhash['page-count'] or self.vhash['page-count'] == '0':
						self.vhash['page-count'] = '?'

					#Send cover
					try:
						fd = CreateCover(self.coverpath,self.vhash)
					except IOError,msg:
						self.filerror = IOError
						raise self.filerror, msg

					s = self.ftp.storbinary('STOT',fd,10240)
					#Return tmpfile name
					covername = s.split()[-1][:-2]

				if self.filepaths:
					filelist = []
					for fil in self.filepaths:
						f = file(fil)
						
						if f.read(4) == "%!PS":
							self.ftp.sendcmd('FORM PS')
						else:
							self.ftp.sendcmd('FORM TIFF')
						f.close()
					
						try:
							fd = SendFile(fil)
						except IOError,msg:
							self.filerror = IOError
							raise self.filerror, msg
						s = self.ftp.storbinary('STOT',fd,10240)
						#Return tmpfile name
						filelist.append(s.split()[-1][:-2])
					self.setparm('document',filelist)
				else:
					if self.parmhash.has_key('DOCUMENT'):
						del self.parmhash['DOCUMENT']

				self.setparm('vres','196')
				#self.setparm('pagewidth','209')
				#self.setparm('pagelength','296')
				self.setparm('pagechop','default')
				self.setparm('chopthreshold','3')

				jobl = []
				jn = len(self.sendhash)
				try:
					for k in self.sendhash.keys():
						fn = self.sendhash[k]['facsimiletelephonenumber']
						pbar.set_mesg('%i job to queue...' % jn)
						pbar + 50
						jn = jn - 1
						self.setparm('dialstring',fn)
						
						self.ftp.sendcmd('JNEW')

						#Send coverpage first
						if self.coverpath:
							if self.cover_as_document:
								self.ftp.sendcmd('JPARM DOCUMENT %s' % covername)
							else:
								self.ftp.sendcmd('JPARM COVER %s' % covername)

						for jparm in self.parmhash.keys():
							if type(self.parmhash[jparm]) == type([]):
								for p in self.parmhash[jparm]:
									self.ftp.sendcmd('JPARM %s %s' % (jparm,p) )
							else:
								self.ftp.sendcmd('JPARM %s %s' % (jparm,self.parmhash[jparm]) )
						jobn = self.ftp.sendcmd('JSUBM')
						jobl.append(string.split(jobn)[2])
						self.ftp.sendcmd('JOB DEFAULT')
					self.ftp.quit()

				except ftplib.error_perm, msg:
					pbar.reset()
					self.ftpexcpt = ftplib.error_perm
					raise self.ftpexcpt , msg
				except ftplib.error_temp, msg:
					pbar.reset()
					self.ftpexcpt = ftplib.error_temp
					raise self.ftpexcpt , msg

				pbar.reset()
				return jobl


def getalljob(inst):
	jobl = []
	for l in inst.getsendq():
		splitted = string.split(l)
		if splitted:
			jobl.append(splitted[0])
	return jobl

  
class SendFile:

	i = 0
	steps = 0
	filesize = 0

	def __init__(self,filename):
		self.filesize = os.stat(filename)[6]
		self.ft = open(filename,'rb')
		pbar.show_percent = 0
		pbar.set_mesg('Transferring fax file...')
	
	def read(self,readbuf):
		buf = self.ft.read(readbuf)
		pbarstep = (self.filesize/readbuf) / 50
		
		if buf:
			if self.steps < 50 and pbarstep and not self.i % pbarstep: 
				pbar + 1
				self.steps = self.steps + 1
			self.i = self.i + 1
			return buf
		else:
			self.ft.close()
			pbar.reset()
	
 
class CreateCover:

	#Fax header
	coverheader = ''
	coverbuf = []
	pbarstep = 0
	steps = 0
	i = 0
	
	def __init__(self,coverfile,phash):
		ft = open(coverfile,'rb')
		tlines = ft.readlines()
		ft.close()
		self.coverbuf = self.get_coverheader(phash) + tlines
		self.coverbuf.reverse()
		self.i = len(self.coverbuf)
		self.pbarstep = self.i / 50
		pbar.show_percent = 0
		pbar.set_mesg('Transferring cover file...')
	
	def get_coverheader(self,shash):
		for k in shash.keys():
			if k == 'comments':
				comments = []
				for l in string.split(shash[k],'\n'):
					comments.append(self.strtohex(l))
			else:
				shash[k] = self.strtohex(shash[k])

		while len(comments) < 20:
			comments.append('')
		return ["""%%!PS-Adobe-2.0 EPSF-2.0
%%%%Creator: Python Class
%%%%Title: Python Class for Hylafax
%%%%Creation Date: Fri Aug 09 20:52:55 2002 
%%%%Origin: 0 0
%%%%BoundingBox: 0 0 592 839
%%%%Pages: 1 ,1
%%%%EndComments
%%%%Begin Prolog
100 dict begin
/to <%s> def
/to-company <%s> def
/to-location () def
/to-voice-number () def
/to-fax-number () def
/pageWidth 0 def
/pageWidth 0 def
/from <%s> def
/from-fax-number () def
/from-voice-number () def
/from-company () def
/from-location () def
/page-count <%s> def
/todays-date <%s> def
/regarding <%s> def
/comments0 <%s> def
/comments1 <%s> def
/comments2 <%s> def
/comments3 <%s> def
/comments4 <%s> def
/comments5 <%s> def
/comments6 <%s> def
/comments7 <%s> def
/comments8 <%s> def
/comments9 <%s> def
/commentsa <%s> def
/commentsb <%s> def
/commentsc <%s> def
/commentsd <%s> def
/commentse <%s> def
/commentsf <%s> def
/commentsg <%s> def
/commentsh <%s> def
/commentsi <%s> def
/commentsl <%s> def
%%%%EndProlog
%%%%Page 1 +1
/wordbreak ( ) def
/BreakIntoLines
{ /proc exch def
  /linewidth exch def
  /textstring exch def
  /breakwidth wordbreak stringwidth pop def
  /curwidth 0 def
  /lastwordbreak 0 def
  /startchar 0 def
  /restoftext textstring def
  { restoftext wordbreak search
    { /nextword exch def pop
      /restoftext exch def
      /wordwidth nextword stringwidth pop def
      curwidth wordwidth add linewidth gt
      { textstring startchar
        lastwordbreak startchar sub
        getinterval proc
        /startchar lastwordbreak def
        /curwidth wordwidth breakwidth add def }
      { /curwidth curwidth wordwidth add
        breakwidth add def
		 } ifelse
      /lastwordbreak lastwordbreak
      nextword length add 1 add def
    }
    { pop exit }
    ifelse
  } loop
  /lastchar textstring length def
  textstring startchar lastchar startchar sub
  getinterval proc
} def

""" % (shash['to'],
	shash['to-company'],
	shash['from'],
	shash['page-count'],
	shash['todays-date'],
	shash['regarding'],
	comments[0],
	comments[1],
	comments[2],
	comments[3],
	comments[4],
	comments[5],
	comments[6],
	comments[7],
	comments[8],
	comments[9],
	comments[10],
	comments[11],
	comments[12],
	comments[13],
	comments[14],
	comments[15],
	comments[16],
	comments[17],
	comments[18],
	comments[19])]
	
	def read(self,ignore=None):
		if self.i:
			self.i = self.i - 1
			if self.steps < 50 and self.pbarstep and not self.i % self.pbarstep: 
				pbar + 1
				self.steps = self.steps + 1
			return self.coverbuf[self.i]
		else:
			pbar.reset()
	 
	def readlines(self):
		self.coverbuf.reverse()
		return self.coverbuf

	def strtohex(self,str):
		strhex = ''
		for c in str:
			strhex += '%02x' % ord(c)
		return strhex

if __name__ == '__main__': #This is a script
	h = hylafax('172.16.1.7')
	h.username = "gigi"
	print h.getstatus()
