#!/usr/bin/python
################################################################################
#	 This is a Pyla form module.
#    Copyright (C) 2000 Andrea Nicolini
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#	 Andrea Nicolini's Address is:
#	 
#		Via G.Zoccoli n.80 Baggiovara 
#		Modena, ITALY 
#		E-Mail <andrea@teamsw.it>
#
################################################################################

import sys,string

try:
	import ldap
except:
	try:
		import winldap
		ldap = winldap
	except:
		try:
			import winldap2
			ldap = winldap2
		except:
			import winldap3
			ldap = winldap3

class DirClass:

	username = ''
	password = ''
	ldaphost = ''
	ldapport = 0
	baseDN   = ''
	ldapobj  = None
	exception = Exception

	def __init__(self,host='',port=389,debug=0):
		self.debug = debug
		self.sethost(host,port)

	def sethost(self,host,port=389):
		hosterr = "Parameter 'host' must be a string."
		porterr = "Parameter 'port' must be an integer."
		if type(host) == type(""):
			self.ldaphost = host
		else:
			raise hosterr
		if type(port) == type(1):
			self.ldapport = port
		else:
			raise porterr

	def login(self,user='',passwd='',basedn=''):
		self.baseDN = basedn
		try:
			self.ldapobj = ldap.open(self.ldaphost,self.ldapport)
			self.ldapobj.simple_bind(user,passwd)
		except ldap.LDAPError, err:
			if self.debug: print err
			raise self.exception , err

	def logout(self):
		try:
			self.ldapobj.unbind_s()
		except ldap.LDAPError, err:
			if self.debug: print err
			
	def query(self,filter='',basedn='',scope=3,attr=None,limit=0):

		if scope == 1:
			scp = ldap.SCOPE_BASE
		elif scope == 2 :
			scp = ldap.SCOPE_ONELEVEL
		else:
			scp = ldap.SCOPE_SUBTREE

		if not basedn: basedn = self.baseDN

		results = None
		filter = self.utfencode(filter)
		basedn = self.utfencode(basedn)
	
		try:
			results = self.ldapobj.search(basedn,scp,filter,attr)
		except ldap.LDAPError, err:
			raise self.exception , err
		newres = []
		
		i = 1
		while 1:
			t,r = self.ldapobj.result(results,0)
			if t == None:
				continue
			elif t == ldap.RES_SEARCH_RESULT:
				break
			elif t == ldap.RES_SEARCH_ENTRY:
				r = r[0]
				dn = self.utfdecode(r[0])
				self.hdecode(r[1])
				data = r[1]
				newres.append((dn,data))
				i += 1

			if limit and i > limit: 
				break
			
		self.ldapobj.abandon(results)
		return newres

	
	def add(self,dn,ml):
		try:
			self.ldapobj.add_s(dn,self.addModlist(ml))
		except ldap.LDAPError, err:
			if self.debug: print err
			return err

	def modify(self,dn,ml):
		try:
			self.ldapobj.modify_s(dn,ml)
		except ldap.LDAPError, err:
			if self.debug: print err
			return err

	def modify2(self,dn,old_modlist,new_modlist):
		try:
			self.ldapobj.modify_s(dn,self.modifyModlist(old_modlist,new_modlist))
		except ldap.LDAPError, err:
			if self.debug: print err
			return err

	def delete(self,dn):
		try:
			self.ldapobj.delete_s(dn)
		except ldap.LDAPError, err:
			if self.debug: print err
			return err

	
	def addModlist(self,entry,ignore_attr_types=[]):
	  """Build modify list for call of method LDAPObject.add()"""
	  ignore_attr_types = map(string.lower,ignore_attr_types)
	  modlist = []
	  for attrtype in entry.keys():
	    if string.lower(attrtype) in ignore_attr_types:
	      # This attribute type is ignored
	      continue
	    # Eliminate empty attr value strings in list
	    attrvaluelist = filter(None,entry[attrtype])
	    if attrvaluelist:
	      modlist.append((attrtype,entry[attrtype]))
	  return modlist
	
	
	def modifyModlist(self,old_entry,new_entry,ignore_attr_types=[],ignore_oldexistent=0):
	  """
	  Build differential modify list for calling LDAPObject.modify()/modify_s()
	
	  old_entry
	      Dictionary holding the old entry
	  new_entry
	      Dictionary holding what the new entry should be
	  ignore_attr_types
	      List of attribute type names to be ignored completely
	  ignore_oldexistent
	      If non-zero attribute type names which are in old_entry
	      but are not found in new_entry at all are not deleted.
	      This is handy for situations where your application
	      sets attribute value to '' for deleting an attribute.
	      In most cases leave zero.
	  """
	  ignore_attr_types = map(string.lower,ignore_attr_types)
	  modlist = []
	  attrtype_lower_map = {}
	  for a in old_entry.keys():
	    attrtype_lower_map[string.lower(a)]=a
	  for attrtype in new_entry.keys():
	    attrtype_lower = string.lower(attrtype)
	    if attrtype_lower in ignore_attr_types:
	      # This attribute type is ignored
	      continue
	    new_value = filter(None,new_entry[attrtype])
	    if attrtype_lower_map.has_key(attrtype_lower):
	      old_value = old_entry.get(attrtype_lower_map[attrtype_lower],[])
	      del attrtype_lower_map[attrtype_lower]
	    else:
	      old_value = []
	    if not old_value and new_value:
	      # Add a new attribute to entry
	      modlist.append((ldap.MOD_ADD,attrtype,new_value))
	    elif old_value and new_value:
	      new_value.sort() ; old_value.sort()
	      if new_value!=old_value:
	        # Replace attribute value of existing attribute
	        # modify an existing attribute
	        modlist.append((ldap.MOD_REPLACE,attrtype,new_value))
	    elif old_value and not new_value:
	      # delete an existing attribute because attribute
	      # value list is empty
	      modlist.append((ldap.MOD_DELETE,attrtype,old_value))
	  if not ignore_oldexistent:
	    # Remove all attributes of old_entry which are not present
	    # in new_entry at all
	    for a in attrtype_lower_map.keys():
	      if a in ignore_attr_types:
	        # This attribute type is ignored
	        continue
	      attrtype = attrtype_lower_map[a]
	      modlist.append((ldap.MOD_DELETE,attrtype,old_entry[attrtype]))
	  return modlist
			
	def dnescape(self,str):
		str = string.strip(str)
		if str:
			for c in '\\,+"<>;':
				str = string.replace(str,c,'\\'+c)
			if str[0] == '#':
				str = '\\' + str
		return str
	
	def qescape(self,str):
		str = string.strip(str)
		if str:
			str = string.replace(str,'\\','\\5C')
			str = string.replace(str,'*','\\2A')
			str = string.replace(str,'(','\\28')
			str = string.replace(str,')','\\29')
		return str

	def makedn(self,attrlist,*vallist):
		str = ''
		if type(attrlist) == type(''):
			attrlist = [attrlist]
		i = 0
		for k in attrlist:
			str = str + k + '=' + self.dnescape(vallist[i]) + ','
			i += 1
		str = str + self.baseDN
	
		dn = self.utfencode(str)
		return dn

	def hencode(self,hash):

		for k in hash.keys():
			if type(hash[k]) == type([]):
				hash[k] = map(self.utfencode,hash[k])
			else:
				hash[k] = self.utfencode(hash[k])
	
	def hdecode(self,hash):
		for k in hash.keys():
			if type(hash[k]) == type([]):
				hash[k] = map(self.utfdecode,hash[k])
			else:
				hash[k] = self.utfdecode(hash[k])
						
	def utfencode(self,str):
		if type(str) != type(''):
			return str.encode('utf-8')
		str = string.strip(str)
		str = unicode(str,'iso-8859-1').encode('utf-8')
		return str

	def utfdecode(self,str):
		if type(str) == type(''):
			return str
		str = string.strip(str)
		str = unicode(str,'utf-8').encode('iso-8859-1')
		return str

 	
class LdapDB(DirClass):
	
	def __init__(self,host='',port=389,debug=0):
		DirClass.__init__(self,host,port,debug)
	
	def addrecord(self,hash):
			
		nhash = {}
		nhash['cn'] = string.strip(hash['cn'])
		nhash['sn'] = nhash['cn']
		nhash['objectClass'] = ['inetorgperson']
		nhash['facsimileTelephoneNumber'] = map(string.strip,filter(None,hash['facsimiletelephonenumber']))
		nhash['mail'] = map(string.strip,filter(None,hash['mail']))

		dn = self.makedn('cn',nhash['cn'])
		self.hencode(nhash)
		ret = self.add(dn,nhash) 
		if self.debug: print ret
		return ret
			
		
	def modrecord(self,dn,hash):
		modlist = []
		self.hencode(hash)
		for k in hash.keys():
			if type(hash[k]) == type([]):
				newvalue = map(string.strip,filter(None,hash[k]))
			else:
				newvalue = string.strip(hash[k])
			modlist.append((ldap.MOD_REPLACE,k,newvalue))
		ret = self.modify(dn,modlist) 
		if self.debug: print ret
		return ret

	
